@@ -18,7 +18,6 @@ import numpy        as np
 import numpy.linalg as npla
 
 import        fsl.data.image as fslimage
-import        fsl.data.mesh  as fslmesh
 import        fsleyes_props  as props
 from . import group          as dcgroup
 
@@ -95,11 +94,11 @@ class DisplayContext(props.SyncableHasProperties):
 
 
     worldLocation = props.Point(ndims=3)
-    """The location property contains the currently selected 3D location (xyz)
-    in the world coordinate system. Whenever the :attr:`location` changes, it
-    gets transformed into the world coordinate system, and propagated to this
-    property. The location of different ``DisplayContext`` instances is
-    synchronised through this property.
+    """The :attr:`location` property contains the currently selected 3D
+    location (xyz) in the current display coordinate system. Whenever the
+    :attr:`location` changes, it gets transformed into the world coordinate
+    system, and propagated to this property. The location of different
+    ``DisplayContext`` instances is synchronised through this property.
 
     .. note:: If any :attr:`.NiftiOpts.transform` properties have been modified
               independently of the :attr:`displaySpace`, this value will be
@@ -107,14 +106,6 @@ class DisplayContext(props.SyncableHasProperties):
     """
 
 
-    vertexIndex = props.Int()
-    """This property may be used to control the :attr:`location` when the
-    currently selected overlay is a :class:`.Mesh`, which comprises
-    a list of vertices. If this property is set to an index into the mesh
-    vertex list, the :attr:`location` will be set to that vertex.
-    """
-
-
     bounds = props.Bounds(ndims=3)
     """This property contains the min/max values of a bounding box (in display
     coordinates) which is big enough to contain all of the overlays in the
@@ -358,10 +349,6 @@ class DisplayContext(props.SyncableHasProperties):
                              self.__name,
                              self.__worldLocationChanged,
                              immediate=True)
-            self.addListener('vertexIndex',
-                             self.__name,
-                             self.__vertexIndexChanged,
-                             immediate=True)
 
         # The overlayListChanged method
         # is important - check it out
@@ -372,8 +359,14 @@ class DisplayContext(props.SyncableHasProperties):
 
     def __del__(self):
         """Prints a log message."""
-        if log:
-            log.debug('{}.del ({})'.format(type(self).__name__, id(self)))
+
+        # The log atribute, and internals of the
+        # logging module, be GC'd before this
+        # method is called, so absorb any errors.
+        try:
+            log.debug('%s.del (%s)', type(self).__name__, id(self))
+        except Exception:
+            pass
 
 
     def destroy(self):
@@ -400,12 +393,16 @@ class DisplayContext(props.SyncableHasProperties):
             self.removeListener('displaySpace',       self.__name)
             self.removeListener('location',           self.__name)
             self.removeListener('worldLocation',      self.__name)
-            self.removeListener('vertexIndex',        self.__name)
+        else:
+            for g in list(self.overlayGroups):
+                self.overlayGroups.remove(g)
+                g.destroy()
 
         for overlay, display in displays.items():
             display.destroy()
 
 
+    @property
     def destroyed(self):
         """Returns ``True`` if this ``DisplayContext`` has been, or is being,
         destroyed, ``False`` otherwise.
@@ -413,7 +410,7 @@ class DisplayContext(props.SyncableHasProperties):
         return self.__overlayList is None
 
 
-    def getDisplay(self, overlay, overlayType=None):
+    def getDisplay(self, overlay, **kwargs):
         """Returns the :class:`.Display` instance for the specified overlay
         (or overlay index).
 
@@ -428,16 +425,14 @@ class DisplayContext(props.SyncableHasProperties):
         :arg overlay:     The overlay to retrieve a ``Display`` instance for,
                           or an index into the ``OverlayList``.
 
-        :arg overlayType: If a ``Display`` instance for the specified
-                          ``overlay`` does not exist, one is created - in
-                          this case, the specified ``overlayType`` is passed
-                          to the :class:`.Display` constructor.
+        All other keyword arguments are assumed to be ``name=value`` pairs,
+        containing initial property values.
         """
 
         if overlay is None:
             raise ValueError('No overlay specified')
 
-        if self.destroyed():
+        if self.destroyed:
             raise ValueError('DisplayContext has been destroyed')
 
         if overlay not in self.__overlayList:
@@ -455,9 +450,7 @@ class DisplayContext(props.SyncableHasProperties):
             if not self.__child:
                 dParent = None
             else:
-                dParent = self.getParent().getDisplay(overlay, overlayType)
-                if overlayType is None:
-                    overlayType = dParent.overlayType
+                dParent = self.getParent().getDisplay(overlay, **kwargs)
 
             from .display import Display
 
@@ -465,13 +458,13 @@ class DisplayContext(props.SyncableHasProperties):
                               self.__overlayList,
                               self,
                               parent=dParent,
-                              overlayType=overlayType)
+                              **kwargs)
             self.__displays[overlay] = display
 
         return display
 
 
-    def getOpts(self, overlay, overlayType=None):
+    def getOpts(self, overlay):
         """Returns the :class:`.DisplayOpts` instance associated with the
         specified overlay.  See :meth:`getDisplay` and :meth:`.Display.opts`
         for more details.
@@ -480,14 +473,14 @@ class DisplayContext(props.SyncableHasProperties):
         if overlay is None:
             raise ValueError('No overlay specified')
 
-        if self.destroyed():
+        if self.destroyed:
             raise ValueError('DisplayContext has been destroyed')
 
         if overlay not in self.__overlayList:
             raise InvalidOverlayError('Overlay {} is not in '
                                       'list'.format(overlay.name))
 
-        return self.getDisplay(overlay, overlayType).opts
+        return self.getDisplay(overlay).opts
 
 
     def getReferenceImage(self, overlay):
@@ -760,13 +753,13 @@ class DisplayContext(props.SyncableHasProperties):
         # for every overlay in the list
         for overlay in self.__overlayList:
 
-            ovlType = self.__overlayList.initOverlayType(overlay)
+            initProps = self.__overlayList.initProps(overlay)
 
             # The getDisplay method
             # will create a Display object
             # if one does not already exist
             new     = overlay not in self.__displays
-            display = self.getDisplay(overlay, ovlType)
+            display = self.getDisplay(overlay, **initProps)
             opts    = display.opts
 
             # Register a listener on the overlay type,
@@ -1176,28 +1169,6 @@ class DisplayContext(props.SyncableHasProperties):
         self.__propagateLocation('display')
 
 
-    def __vertexIndexChanged(self, *a):
-        """Called when the :attr:`vertexIndex` property changes. Propagates
-        the new location to the :attr:`location` property.
-        """
-
-        vidx = self.vertexIndex
-        ovl  = self.getSelectedOverlay()
-
-        # If the current overlay is a mesh, and
-        # the index looks valid, propagate it on
-        # to the location.
-        if isinstance(ovl, fslmesh.Mesh) and \
-           vidx >= 0                     and \
-           vidx < ovl.vertices.shape[0]:
-
-            opts = self.getOpts(ovl)
-            loc  = ovl.vertices[vidx, :]
-            loc  = opts.transformCoords(loc, 'mesh', 'display')
-
-            self.location.xyz = loc
-
-
     def __propagateLocation(self, dest):
         """Called by the :meth:`__locationChanged` and
         :meth:`__worldLocationChanged` methods. The ``dest`` argument may be
