@@ -514,7 +514,7 @@ There are two requirements that must be met:
     the parser. This can easily be accomplished simply by using the expression
     somewhere in a comment, as in the example above.
 
-  - The only expression types that you cannot pass direcrly into a routine
+  - The only expression types that you cannot pass directly into a routine
     are Parameters with a length greater than 1 (e.g. matrix parameters). For
     example, this will result in a compiler error::
 
@@ -537,6 +537,22 @@ The ``arb_call`` function requires:
   - Mappings between the variables in your code, and the routine's input and
     output parameters. These must all be passed as named (keyword) arguments.
     You cannot use swizzle masks in these mappings.
+
+
+Temporary variables in included files
+-------------------------------------
+
+
+When your shader logic is split across several files, it can become difficult
+to ensure that you don't introduce any variable name conflicts. To avoid this,
+the ``arb_temp`` macro allows you to safely define temporary variables in
+included files. For example::
+
+    {{ arb_temp('mytemp') }}
+    MOV {{ mytemp }}, 1;
+
+The ``arb_temp`` macro will assign a unique variable name, ensuring that there
+are no conflicts with other files.
 """
 
 
@@ -635,6 +651,14 @@ def fillARBP(vertSrc,
                         files that may be included in the given source files.
     """
 
+    vertParams    = dict(vertParams)
+    vertParamLens = dict(vertParamLens)
+    fragParams    = dict(fragParams)
+    fragParamLens = dict(fragParamLens)
+    constants     = dict(constants)
+    textures      = dict(textures)
+    attrs         = dict(attrs)
+
     vertVars = _findDeclaredVariables(vertSrc)
     fragVars = _findDeclaredVariables(fragSrc)
 
@@ -786,7 +810,7 @@ def _render(src, env, includePath):
         # 1. Looks up the called filename in the includes dictionary
         # 2. Resolve any arguments which need to be passed through
         #    from an outer arb_call
-        # 3. Generates code - render file source with the probvided
+        # 3. Generates code - render file source with the provided
         #    argument mappings
         # 3. Return generated code
 
@@ -808,7 +832,7 @@ def _render(src, env, includePath):
             for k, v in args.items():
                 if not isinstance(v, six.string_types):
                     continue
-                match = passThroughExpr.fullmatch(v)
+                match = passThroughExpr.match(v)
                 if match:
                     callEnv[k] = oce[match.group(1).strip()]
 
@@ -818,9 +842,9 @@ def _render(src, env, includePath):
         oce = dict(args)
         oce.update(temps)
 
-        outerCallArgs[0] = oce
-        source           = callTemplate.render(**callEnv)
-        outerCallArgs[0] = None
+        outerCallArgs.insert(0, oce)
+        source = callTemplate.render(**callEnv)
+        outerCallArgs.pop(0)
 
         sourceLines.extend(source.split('\n'))
         source = '\n'.join(sourceLines)
